# $Id: StochasticDAGGenerator.py,v 1.1 2005/11/10 20:32:55 kipp Exp $

#
# Preamble.
#

from glue import segments
import onasys.DAGGenerator
from onasys import S5DAGGeneratorA

#
# A DAG generator plug-in for making a single DAG that spans multiple
# segments.
#
# How to use this.  This DAG generator plug-in has a [daggenerator] section
# in the .ini file whose contents are identical to the [daggenerator]
# section used by the S5DAGGeneratorA plug-in.
#
# How this works.  This is based off of S5DAGGeneratorA.  The Config class
# for this plug-in is simply a copy of that plug-in's Config class (thus
# the .ini file is the same).  That plug-in defines a
# DAGGeneratorParameters class describing a DAG for a single segment
# (actually, just the start and stop times for the segment).  We define a
# DAGGeneratorParameters class for *this* plug-in as a list of
# DAGGeneratorParameters objects from the S5DAGGeneratorA plug-in.  We call
# that plug-in to generate its list, and then stick the entire list into
# one of our DAGGeneratorParameters objects.  We return that object as the
# single object in our list of DAGs returned to the daemon.
#

class DAGGeneratorConfig(S5DAGGeneratorA.DAGGeneratorConfig):
	pass

class DAGGeneratorParameters(onasys.DAGGenerator.DAGGeneratorParameters):
	def __init__(self, list):
		self.list = list
		self.extent = segments.segmentlist([dag.dataseg for dag in list]).extent()

	def __repr__(self):
		return repr(self.list)

def DAGGeneratorParameterList(clock, config, available, used):
	# Ask S5DAGGeneratorA to make a list of DAGs.
	list = S5DAGGeneratorA.DAGGeneratorParameterList(clock, config, available, used)

	# Put that list into our DAGGeneratorParameters object and return
	# a list containing that one object back to the daemon.
	return [DAGGeneratorParameters(list)]

class DAGGenerator(onasys.DAGGenerator.DAGGenerator):
	def dag_name(self):
		return self.tag + "-" + str(int(self.params.extent[0])) + "-" + str(int(self.params.extent.duration())) + ".dag"

	def generate(self):
		# FIXME
		pass
